import datetime
from flask import Blueprint, request, jsonify, make_response, json
from app.patients.schemas import patient_schema
from app.utils import add_logo, get_logo_from_logotable
from app.patients.models import Patient
from app.rooms.models import Room
from app.devices.models import Device
from sqlalchemy import desc
from app import db
from app.utils import token_required, paginated_query, delete_record, single_record_query

patient = Blueprint('patient', __name__)
@patient.route('/api/v1/add/patient', methods=['POST'])
@token_required(1, 2, 3)
def create_patient(current_user):
    try:
        patient_dict = json.loads(request.form.get('formData'))
        if patient_dict['device_name'] == "Device not available":
            patient_dict['device_name'] = ""
        loaded_patient_data = patient_schema.load({
            'site_id': request.args.get('site_id'),
            'device_id': str(patient_dict['device_id']),
            'device_name': patient_dict['device_name'].strip(),
            'patient_name': patient_dict['patient_name'].strip(),
            'logo_id': add_logo(request.files.get('image')),
            'gender': patient_dict['gender'].capitalize(),
            'dob': patient_dict['dob'] if patient_dict.get('dob') else datetime.datetime.now().strftime('%Y-%m-%d'),
        })
        loaded_patient_data.pop('device_name')
        db.session.add(Patient(**loaded_patient_data))
        db.session.commit()
        return make_response(jsonify({'message': 'Patient added successfully', "status": "Success"}), 200)
    except Exception as e:
        return make_response(jsonify({'args': e.args, "message": 'failed added Patient', "status": "error"}), 500)
    
@patient.route('/patient_name', methods=['GET'])
@token_required(1, 2, 3)
def patient_name(current_user):
    try:
        search = request.args.get('search')
        site_id = request.args.get('site_id')
        patients_query = Patient.query
        if site_id:
            patients_query = patients_query.filter_by(site_id=site_id)
        if search:
            patients_query = patients_query.filter(Patient.patient_name.ilike(f'%{search}%'))
        patients = patients_query.all()
        serialized_patients = []
        for patient in patients:
            if patient.patient_name is None:
                return jsonify({"error": "Patient name is required"}), 400
            data = get_logo_from_logotable(patient.logo_id) if patient.logo_id else ("", "")
            serialized_patients.append({
                'id':patient.patient_id,
                'name': patient.patient_name,
                'room_id': patient.room_id,
                'room_name': Room.query.filter_by(id=patient.room_id).first().room_name,
                'logo': data[0],
                'logo_content_type': data[1],
            })
        return jsonify({"patient": serialized_patients}), 200
    except Exception as e:
        return make_response(jsonify({"Error": str(e)}), 500)

@patient.route('/api/v1/get/patient', methods=['GET'])
@token_required(1, 2, 3)
def get_patient(current_user):
    try:
        patient_query = Patient.query.order_by(desc(Patient.patient_id))
        patients_paginated, data = paginated_query(patient_query, [Patient.patient_name], Patient.site_id)
        serialized_patients = []
        for patient in patients_paginated:
            logo_data = get_logo_from_logotable(patient.logo_id) if patient.logo_id else ("", "")
            device_name = ""
            room_name = ""
            device = Device.query.filter_by(device_id=patient.device_id).first()
            if device:
                device_name = device.device_name
                room = Room.query.filter_by(id=device.room_id).first()
                if room:
                    room_name = room.room_name
            serialized_patients.append({
                'id': patient.patient_id,
                'site_id': patient.site_id,
                'patient_name': patient.patient_name,
                'device_id': patient.device_id,
                'room_name': room_name,
                'device_name': device_name,
                'gender': patient.gender,
                'dob': patient.dob,
                'logo': logo_data[0],
                'logo_content_type': logo_data[1],
            })
        response_data = {'patient': serialized_patients, 'data': data}
        return make_response(jsonify(response_data), 200)
    except Exception as e:
        return make_response(
            jsonify({'message': f'Error: {str(e)}', 'status': 'error'}), 500 )

@patient.route('/api/v1/get/patient_by_id/<int:id>', methods=['GET'])
@token_required(1, 2, 3)
def get_patient_by_id(current_user, id):
    try:
        patient = single_record_query(Patient, Patient.patient_id, id, current_user)
        if patient:
            data = ("", "")
            if patient.logo_id:
                data = get_logo_from_logotable(patient.logo_id) if patient.logo_id else ("", "")
            device = Device.query.filter_by(device_id=patient.device_id).first()
            device_name = device.device_name if device else None
            return make_response(jsonify({'patient': {
                'patient_id': patient.patient_id,
                'site_id': patient.site_id,
                'patient_name': patient.patient_name,
                'device_name': device_name,
                'gender': patient.gender,
                'dob': patient.dob.strftime('%Y-%m-%d')
            }, "logo": {
                'logo': data[0],
                'logo_content_type': data[1],
            }}), 200)
        return make_response(jsonify({'message': 'Patient not found', "status": "error"}), 500)
    except Exception as e:
        return make_response(jsonify({'message': f'error: {e}', "status": "error"}), 500)
 
@patient.route('/api/v1/update_patient/<int:patient_id>', methods=['PUT'])
@token_required(1, 2, 3)
def update_patient(current_user, patient_id):
    try:
        patient_dict = json.loads(request.form.get('formData'))
        patient_to_update = single_record_query(Patient, Patient.patient_id, patient_id, current_user)
        if not patient_to_update:
            return make_response(jsonify({'message': 'Patient not found', "status": "error"}), 404)
        if patient_dict['device_name'] == "Device not available" or patient_dict['device_name'] == None:
            patient_dict['device_name'] = ""
        for key, value in patient_schema.load({
            'site_id': request.args.get('site_id'),
            'patient_name': patient_dict['patient_name'].strip(),
            'device_id': str(patient_dict.get('device_id', patient_to_update.device_id)),
            'gender': patient_dict.get('gender', patient_to_update.gender).capitalize(),
            'dob': patient_dict.get('dob', patient_to_update.dob),
            'device_name': patient_dict.get('device_name'),
            'logo_id': add_logo(request.files['image']) if 'image' in request.files else patient_to_update.logo_id
        }).items():
            setattr(patient_to_update, key, value)
        db.session.commit()
        return make_response(jsonify({'message': 'Patient updated successfully', "status": "Success"}), 200)
    except Exception as e:
        return make_response(jsonify({'args': e.args, "message": 'Failed to update Patient', "status": "error"}), 500)
@patient.route('/api/v1/delete-patient/<id>', methods=['DELETE'])
@token_required(1, 2, 3)
def delete_patient(current_user, id):
    return delete_record(Patient, current_user, id, "Patient", "patient_id")